@@ -375,6 +375,10 @@ class Session(object):
                 tmpsocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         else:
             tmpsocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        try:
+            tmpsocket.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 16777216)
+        except Exception:
+            pass
         if server is None:
             # Rather than wait until send() to bind, bind now so that we have
             # a port number allocated no matter what
@@ -583,9 +587,18 @@ class Session(object):
             self.broken = True
             if self.socket:
                 self.socketpool[self.socket] -= 1
+        while self.logonwaiters:
+            waiter = self.logonwaiters.pop()
+            try:
+                waiter({'error': 'Session failed to initialize'})
+            except Exception:
+                pass
 
     def onlogon(self, parameter):
         if 'error' in parameter:
+            while self.logonwaiters:
+                waiter = self.logonwaiters.pop()
+                waiter(parameter)
             self._mark_broken(parameter['error'])
         elif self.onlogpayload:
             self._cmdwait()
@@ -1763,10 +1776,11 @@ class Session(object):
                 self._mark_broken()
                 return
             else:
+                self.maxtimeout = 2
                 if self.sessioncontext == 'ESTABLISHED':
                     self.onlogpayload = self.lastpayload
                     self.onlogpayloadtype = self.last_payload_type
-                self.maxtimeout = 6
+                    self.maxtimeout = 6
                 self._relog()
                 return
         elif self.sessioncontext == 'FAILED':
