@@ -24,6 +24,7 @@ import json
 import socket
 import ssl
 import threading
+import os
 
 import six
 
@@ -97,6 +98,8 @@ class FileDownloader(threading.Thread):
 
 
 def get_upload_form(filename, data, formname, otherfields, boundary=BND):
+    if not boundary:
+        boundary = base64.b64encode(os.urandom(54))[:70]
     ffilename = filename.split('/')[-1]
     if not formname:
         formname = ffilename
@@ -124,8 +127,8 @@ def get_upload_form(filename, data, formname, otherfields, boundary=BND):
                       formname, ffilename).encode('utf-8'))
         form += b'Content-Type: application/octet-stream\r\n\r\n' + data
         form += b'\r\n--' + boundary + b'--\r\n'
-        uploadforms[filename] = form
-        return form
+        uploadforms[filename] = form, boundary
+        return uploadforms[filename]
 
 
 class SecureHTTPConnection(httplib.HTTPConnection, object):
@@ -142,6 +145,7 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
         self.broken = False
         self.thehost = host
         self.theport = port
+        self._upbuffer = None
         try:
             httplib.HTTPConnection.__init__(self, host, port, strict=strict,
                                             **kwargs)
@@ -332,16 +336,16 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
                      the file.
         :return:
         """
-        boundary = base64.b64encode(os.urandom(54))[:70]
         if data is None:
             data = open(filename, 'rb')
         ulhdrs = self.stdheaders.copy()
         if formwrap:
-            self._upbuffer = io.BytesIO(get_upload_form(
-                filename, data, formname, otherfields, boundary))
-            ulhdrs['Content-Type'] = b'multipart/form-data; boundary=' + boundary
-            ulhdrs['Content-Length'] = len(uploadforms[filename])
-            self.ulsize = len(uploadforms[filename])
+            guf = get_upload_form(
+                filename, data, formname, otherfields, boundary=None)
+            self._upbuffer = io.BytesIO(guf[0])
+            ulhdrs['Content-Type'] = b'multipart/form-data; boundary=' + guf[1]
+            ulhdrs['Content-Length'] = len(uploadforms[filename][0])
+            self.ulsize = len(uploadforms[filename][0])
         else:
             canseek = True
             try:
@@ -384,6 +388,8 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
         return body
 
     def get_upload_progress(self):
+        if self._upbuffer is None:
+            return 0.0
         return float(self._upbuffer.tell()) / float(self.ulsize)
 
     def request(self, method, url, body=None, headers=None, referer=None):
